<?php
/**
 * Staffing Engine - Chatbot - Admin class
 *
 * @link        https://staffing-engine-chatbot.ai/
 * @since       0.2.0
 *
 * @package     Staffing Engine - Chatbot
 * @author      Staffing Engine
 * @copyright   (c) 2022 staffing Engine
 *
 * @license     https://opensource.org/licenses/GPL-3.0 GNU General Public License version 3
 */

namespace StaffingEngine\Chatbot;

use StaffingEngine\Chatbot\Bin\RationalOptionPages;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Configure plugin in wp admin
 *
 * @since 0.2.0
 */
final class Admin {

  /**
	 * Instance of this class.
	 *
	 * @since    0.2.0
	 *
	 * @var      object
	 */
	protected static $instance = null;

  /**
	 * Instance Builder
	 *
	 * Singleton pattern means we create only one instance of the class.
	 *
   * @since 0.2.0
   *
   * @return object
   */
  public static function get_instance() {
    if ( ! isset( self::$instance ) ) {
      self::$instance = new self();

      self::$instance->register_hooks();
    }

    return self::$instance;
  }

	/**
	 * Constructor
	 *
	 * @since 0.2.0
	 */
	public function __construct() {}

  /**
   * Register hooks
   *
   * Tell wordpress to enqueue scripts, set up admin pages, and modify the UI
   *
   * @since 0.2.0
   */
  private function register_hooks() {
    $should_notify = $this->should_notify_unsupported_php();
    if ( $should_notify ) {
      add_action( 'admin_notices', array( $this, 'notify_unsupported_php' ) );
    }

    $this->buildSettingsPage();

    add_action( 'admin_enqueue_scripts', array( $this, 'loadSettingPageStyles' ) );
    add_action( 'admin_enqueue_scripts', array( $this, 'loadSettingsPageScripts' ) );

    add_filter( 'plugin_action_links_' . SE_CHAT_BASENAME , array( $this, 'add_plugin_links' ), 10, 1 );
  }

  /**
   * Load settings page styles
   *
   * @since 0.2.0
   */
  public function loadSettingPageStyles($hook) {
    if ( 'settings_page_staffing_engine_chatbot' != $hook ) {
      return;
    }

    wp_enqueue_style( Plugin::SLUG . '-settings-styles' , path_join( SE_CHAT_PUBLIC_ASSET_PATH, 'css/se-chatbot-settings.css'), array(), null );
  }

  /**
   * Load settings page scripts
   *
   * @since 0.2.0
   */
  public function loadSettingsPageScripts($hook) {
    if ( 'settings_page_staffing_engine_chatbot' != $hook ) {
      return;
    }

    wp_enqueue_script( Plugin::SLUG . '-settings-scripts' , path_join( SE_CHAT_PUBLIC_ASSET_PATH, 'js/se-chatbot-settings.js' ), array(), null, true );
  }

  /**
   * Add custom links on the plugins page
   *
   * @param  array  $links
   * @since  0.2.0
   * @return array
   */
  public function add_plugin_links( $links ) {
    $new_links = array(
      'settings' => sprintf( __( '<a href="%s">Settings</a>', Plugin::SLUG ), esc_url( admin_url( 'options-general.php?page=staffing_engine_chatbot' ) ) ),
    );

    return array_merge( $new_links, $links );
  }

  /**
   * Build the settings page UI
   *
   * @since 0.2.0
   */
  private function buildSettingsPage() {
    $pageConfig = array(
			Plugin::SLUG . '_settings'	=> array(
				'parent_slug' => 'options-general.php',
				'page_title'	=> __( Plugin::NAME, Plugin::SLUG ),
        'menu_title' => __( 'Staffing Engine', Plugin::SLUG ),
				'sections' => array(
					'config' => array(
						'title'	 => __( 'Configuration', Plugin::SLUG ),
            'text'   => __( 'Connect the chatbot to our chat services', Plugin::SLUG ),
						'fields' => array(
							'key' => array(
								'id'		  => 'key',
								'type'		=> 'text',
								'title'		=> __( 'Bot key', Plugin::SLUG ),
                'text'    => __( 'Your unique chat instance key. Contact Staffing Engine support if you haven\'t received one.', Plugin::SLUG ),
								'default'	=> '',
							),
            ),
          ),
          'appearance' => array(
            'title'  => __( 'Appearance', Plugin::SLUG),
            'text'   => __( 'Customize the chat widget\'s appearance', Plugin::SLUG ),
            'fields' => array(
              'position' => array(
                'id'		=> 'position',
                'title'	=> __( 'Bot position', Plugin::SLUG ),
                'text'  => __( 'Set the location of the bot on your site.', Plugin::SLUG ),
                'type'  => 'select',
                'value' => 'bottom-right',
                'choices'		=> array(
                  'bottom-right' => __( 'Bottom Right', Plugin::SLUG ),
                  'bottom-left'  => __( 'Bottom Left', Plugin::SLUG ),
                  'top-right'    => __( 'Top Right', Plugin::SLUG ),
                  'top-left'     => __( 'Top Left', Plugin::SLUG ),
                ),
              ),
              'widgetColor'	=> array(
                'id'    => 'widget_color',
                'title'	=> __( 'Widget Color', Plugin::SLUG ),
                'text'  => __( 'Set the background color of the chatbot embed.', Plugin::SLUG ),
                'type'	=> 'color',
                'value'	=> '#c4c4c4',
              ),
              'thumbLogo'	=> array(
                'id'    => 'thumb_logo',
                'title'	=> __( 'Thumb Logo', Plugin::SLUG ),
                'text'  => __( 'Upload a custom logo for your chatbot.', Plugin::SLUG ),
                'type'	=> 'media',
                'value'	=> '',
              ),
              'animation' => array(
                'id'      => 'animation',
                'title'	  => __( 'Animation type', Plugin::SLUG ),
                'text'  => __( 'Choose an animation style to get the attention of site visitors.', Plugin::SLUG ),
                'type'	  => 'select',
                'value'	  => 'pulse',
                'choices'	=> array(
                  'pulse' => __( 'Pulse', Plugin::SLUG ),
                  'ring'  => __( 'Ring', Plugin::SLUG ),
                  'zoom'  => __( 'Zoom', Plugin::SLUG ),
                ),
              ),
            ),
          ),
          'invite_popup' => array(
            'title'  => __( ' Invitation Popup', Plugin::SLUG ),
            'text'   => __( 'Show an informational popup next to the widget after a set duration', Plugin::SLUG ),
            'fields' => array(
              'inviteMessage'	=> array(
                'id'    => 'invite_message',
                'title'	=> __( 'Invite Message', Plugin::SLUG ),
                'type'	=> 'textarea',
                'class' => 'inviteMessage',
                'rows'  => 5,
                'text'	=> __( 'Text shown to promote your chatbot in the popup. Leave empty to not show an invite popup.', Plugin::SLUG ),
                'value'	=> 'Hey! Thanks for stopping by Staffing Engine. 👋 I am the chatbot. I can help you find the right job for you. Just type in a job title or keyword and I will find the best jobs for you. 😊',
              ),
              'inviteButtonText' => array(
                'id'    => 'invite_button_text',
                'title'	=> __( 'Invite Button text', Plugin::SLUG ),
                'text'  => __( 'Display a button with this text in the popup. If blank, no button will be shown.', Plugin::SLUG ),
                'type'  => 'text',
                'value' => '',
              ),
              'inviteImage' => array(
                'id'    => 'invite_image',
                'title'	=> __( 'Invite Image', Plugin::SLUG ),
                'text'  => __( 'Display an image in the popup. If not image is selection, one will not be shown.', Plugin::SLUG ),
                'type'  => 'media',
                'value' => '',
              ),
              'inviteShowCloseIcon' => array(
                'id'      => 'invite_show_close_icon',
                'title'	  => __( 'Show Close Icon', Plugin::SLUG ),
                'text'    => __( 'Show the close icon on the popup. If disabled, it cannot be dismissed.', Plugin::SLUG  ),
                'type'    => 'radio',
                'value' => 'yes',
                'choices'		=> array(
                  'yes'	=> __( 'Show', Plugin::SLUG),
                  'no'	=> __( 'Don\'t Show', Plugin::SLUG ),
                ),
              ),
              'inviteTimeout'	=> array(
                'id'    => 'invite_timeout',
                'title'	=> __( 'Invite Timeout', Plugin::SLUG ),
                'text'  => __( 'Set how long until the popup appears (in seconds)', Plugin::SLUG ),
                'type'	=> 'number',
                'value'	=> 6,
              ),
            ),
          ),
          'behavior' => array(
            'title'  => __( 'Behavior', Plugin::SLUG),
            'text'   => __( 'Customize the chat widget\'s behavior', Plugin::SLUG ),
            'fields' => array(
              'autoExpandDelay'	=> array(
                'id'	  => 'auto_expand_delay',
                'title'	=> __( 'Auto Expand Delay', Plugin::SLUG ),
                'text'  => __( 'How long to wait (in seconds) before the chatbot window automatically opens. Set to 0 to disable auto expand.', Plugin::SLUG ),
                'type'	=> 'number',
                'value'	=> 0,
              ),
              'allowStartNewConversation'		=> array(
                'id'      => 'allow_start_new_conversation',
                'title'   => __( 'Allow starting a new conversation', Plugin::SLUG ),
                'text'    => __( 'Set whether site visitors can start a new conversation or are required to pick up from a previous conversation, if one exists.', Plugin::SLUG ),
                'type'    => 'radio',
                'value' => 'yes',
                'choices'		=> array(
                  'yes'	=> __( 'Allow', Plugin::SLUG),
                  'no'	=> __( 'Don\'t Allow', Plugin::SLUG ),
                ),
              ),
            ),
          ),
				),
			),
		);

    new RationalOptionPages( $pageConfig );
  }

  /**
   * Check if we should notify about an unsupported PHP version
   *
   * @since 0.2.0
   * @return boolean
   */
  private function should_notify_unsupported_php() {
    return version_compare( PHP_VERSION, '7.1.0', '<=' );
  }


  /**
   * Admin Notice if Unsupported PHP Version. PHP Versions below this might have issues running the plugin.
   *
   * @since 0.2.0
   */
  public function notify_unsupported_php() {
    $html = '<div class="notice notice-error is-dismissible"><p>%1$s</p></div>';

    $message = __( '%s requires PHP Version %s or better. Contact your web host to upgrade so you can use the features offered by this plugin.', Plugin::SLUG );

    printf( $html, esc_html( sprintf( $message, Plugin::NAME, '7.1.0' ) ) );
  }
}
