<?php
/**
 * Staffing Engine - Chatbot - Embed class
 *
 * @link            https://staffingengine.ai/
 * @since           0.1.0
 *
 * @package         Staffing Engine Chatbot
 * @subpackage      Core
 * @author          Staffing Engine
 * @copyright       (c) 2022 Staffing Engine
 *
 * @license         https://opensource.org/licenses/GPL-3.0 GNU General Public License version 3
 */

namespace StaffingEngine\Chatbot;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * Render chatbot embed
 *
 * @since 0.1.0
 */
final class Embed {

  /**
	 * Instance of this class.
	 *
	 * @since    0.2.0
	 *
	 * @var      object
	 */
	protected static $instance = null;

  private static $options = array();

  private static $final_options = array();

  /**
	 * Instance Builder
	 *
	 * Singleton pattern means we create only one instance of the class.
	 *
   * @since 0.2.0
   *
   * @return object
   */
  public static function get_instance() {
    if ( ! isset( self::$instance ) ) {
      self::$instance = new self();

      self::$instance->register_hooks();
    }

    return self::$instance;
  }

	/**
	 * Constructor
	 *
	 * @since 0.1.0
	 */
	public function __construct() {
    self::$options = get_option( Plugin::SLUG . '_settings', array() );
    self::$final_options = $this->get_options_with_defaults();


    Utils\debug_log('Chatbot Embed loaded with config:');
    Utils\debug_log(print_r( self::$final_options, true ) );
  }

  /**
   * Register hooks
   *
   * Tell wordpress to enqueue scripts, set up admin pages, and modify the UI
   *
   * @since 0.2.0
   */
  private function register_hooks() {
    add_action( 'wp_footer', array( $this, 'embedTemplate' ) );
    add_action( 'wp_print_footer_scripts', array( $this, 'print_scripts' ) );
    add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
    add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_styles' ) );
  }

	/**
	 * Insert the html template to embed the chatbot
	 *
	 * @since 0.1.0
	 */
	public function embedTemplate() {
		echo'<div id="staffing-engine-chatbot"></div>';
	}

	/**
	 * Print Scripts
	 *
	 * @since 0.1.0
	 */
	public function print_scripts() {
		?>
<script>
  const config = <?php echo json_encode( self::$final_options ); ?>;

  const staffingEngineChatConfig = Object.assign(config, {
    container: '#staffing-engine-chatbot',
    allowChangeChatWindowSize: false,
    allowDrag: false,
    appearance: {
      blurredBackground: false
    },
    pageData: {
      url: window.location.href,
    },
  });
</script>
<?php
	}

	/**
	 * Enqueue Scripts
	 *
	 * @since 0.1.0
	 */
	public function enqueue_scripts() {
		wp_enqueue_script( Plugin::SLUG . '-rwc' , path_join( SE_CHAT_PUBLIC_ASSET_PATH, 'js/richWebChat.umd.min.js' ), array(), null, true );
    wp_enqueue_script( Plugin::SLUG . '-chatbot' , path_join( SE_CHAT_PUBLIC_ASSET_PATH, 'js/se-chatbot.js' ), array(), null, true );
	}
	/**
	 * Enqueue Styles
	 *
	 * @since 0.1.0
	 */
	public function enqueue_styles() {
		wp_enqueue_style( Plugin::SLUG . '-rwc-styles', path_join( SE_CHAT_PUBLIC_ASSET_PATH, 'css/richWebChat.css'), array(), null );
    wp_enqueue_style( Plugin::SLUG . '-chatbot-styles' , path_join( SE_CHAT_PUBLIC_ASSET_PATH, 'css/se-chatbot.css'), array(), null );
	}

  /**
   * Parse user-defined settings and merge with sensible defaults
   *
   * @since 0.2.0
   * @return array
   */
  private function get_options_with_defaults() {
    $optionsSafe = [
      "chatUrl" => 'https://chat.staffingengine.onereach.ai/' . ( array_key_exists( 'key', self::$options ) ? esc_attr( self::$options['key'] ) : '' ),
      "position" => array_key_exists( 'position', self::$options ) ? self::$options['position'] : 'bottom-right',
      "widgetColor" => array_key_exists( 'widget_color', self::$options ) ? esc_attr( self::$options['widget_color'] ) : '#c4c4c4',
      "animation" => array_key_exists( 'animation', self::$options ) ? esc_attr( self::$options['animation'] ) : 'pulse',
      "inviteMessage" => array_key_exists( 'invite_message', self::$options ) ? esc_attr( self::$options['invite_message'] ) : 'Hey! Thanks for stopping by Staffing Engine. 👋 ',
      "inviteButton" => array_key_exists( 'invite_button_text', self::$options ) ? esc_attr( self::$options['invite_button_text'] )  : '',
      "inviteImage" => array_key_exists( 'invite_image', self::$options ) ? esc_attr( self::$options['invite_image'] ) : '',
      "inviteTimeout" => array_key_exists( 'invite_timeout', self::$options ) ? $this->parse_ms_setting(esc_attr( self::$options['invite_timeout'] ) . '000') : 5000,
      "showCloseIcon" => array_key_exists( 'invite_show_close_icon', self::$options ) ? $this->parse_boolean_setting(esc_attr(self::$options['invite_show_close_icon'])) : true,
      "autoExpandDelay" => array_key_exists( 'auto_expand_delay', self::$options ) ? $this->parse_ms_setting(esc_attr( self::$options['auto_expand_delay'] ) . '000') : 0,
      "allowStartNewConversation" => array_key_exists( 'allow_start_new_conversation', self::$options ) ? $this->parse_boolean_setting(esc_attr(self::$options['allow_start_new_conversation'])) : true,
    ];

    $userDefinedLogo = array_key_exists( 'thumb_logo', self::$options ) ? esc_attr( self::$options['thumb_logo'] ) : '';
    if ($userDefinedLogo == '') {
      $optionsSafe["thumbLogo"] = path_join( SE_CHAT_PUBLIC_ASSET_PATH, 'assets/staffing-engine-chatbot_default.png' );
    } else {
      $optionsSafe["thumbLogo"] = $userDefinedLogo;
    }

    return $optionsSafe;
  }

  /**
   * Parse a setting as ms
   *
   * @since 0.1.0
   */
  private function parse_ms_setting($val) {
    if (is_numeric($val)) {
      return $val + 0;
    }
    return 0;
  }

  /**
   * Parse a boolean value. Usually these are radios configured to store 'yes' or 'no'
   *
   * @since 0.1.0
   */
  private function parse_boolean_setting($val) {
    if ($val === 'yes') {
      return true;
    } else if ($val === 'no') {
      return false;
    } else {
      // Default to true if theres some other value
      return true;
    }
  }
}


